﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.UI;

namespace DragDropTreeViewToGrid
{
    public partial class Form1 : Form
    {
        BindingList<Item> treeViewItems = new BindingList<Item>();
        BindingList<Item> gridViewItems = new BindingList<Item>();

        public Form1()
        {
            InitializeComponent();

            for (int i = 0; i < 10; i++)
            {
                treeViewItems.Add(new Item(Guid.NewGuid().ToString(),"Node" + i));
                gridViewItems.Add(new Item(Guid.NewGuid().ToString(),"Item" + i));
            }

            this.radGridView1.DataSource = gridViewItems;
            this.radGridView1.AutoSizeColumnsMode = GridViewAutoSizeColumnsMode.Fill;

            this.radTreeView1.DataSource = treeViewItems;
            this.radTreeView1.DisplayMember = "Title";
            this.radTreeView1.ValueMember = "UniqueIdentifier";

            this.radTreeView1.AllowDragDrop = true;
            this.radTreeView1.MultiSelect = true;
            this.radTreeView1.TreeViewElement.DragDropService.PreviewDragStart += DragDropService_PreviewDragStart;
            this.radTreeView1.TreeViewElement.DragDropService.PreviewDragOver += DragDropService_PreviewDragOver;
            this.radTreeView1.TreeViewElement.DragDropService.PreviewDragDrop += DragDropService_PreviewDragDrop;
        }

        List<RadTreeNode> draggedNodes;

        private void DragDropService_PreviewDragStart(object sender, PreviewDragStartEventArgs e)
        {
            draggedNodes = new List<RadTreeNode>();
            TreeNodeElement draggedNodeElement = e.DragInstance as TreeNodeElement;
            if (draggedNodeElement != null)
            {
                foreach (RadTreeNode node in this.radTreeView1.SelectedNodes)
                {
                    draggedNodes.Add(node);
                }
            }
        }
        
        private void DragDropService_PreviewDragDrop(object sender, RadDropEventArgs e)
        {
            GridDataRowElement targetRowElement = e.HitTarget as GridDataRowElement;
            GridTableElement targetTableElement = e.HitTarget as GridTableElement;

            if (targetRowElement == null && targetTableElement == null)
            {
                return;
            }

            if (draggedNodes == null || draggedNodes.Count == 0)
            {
                return;
            }

            e.Handled = true;         
   
            foreach (RadTreeNode node in draggedNodes)
            {
                Item draggedDataBoundItem = node.DataBoundItem as Item;
                treeViewItems.Remove(draggedDataBoundItem);

                if (targetRowElement != null)
                {
                    int index = GetTargetRowIndex(targetRowElement, e.DropLocation);
                    gridViewItems.Insert(index, draggedDataBoundItem);
                }
                else if (targetTableElement != null)
                {
                    gridViewItems.Add(draggedDataBoundItem);
                }
            }
        }

        private int GetTargetRowIndex(GridDataRowElement row, Point dropLocation)
        {
            int halfHeight = row.Size.Height / 2;
            int index = row.RowInfo.Index;
            if (dropLocation.Y > halfHeight)
            {
                index += 1;
            }
            return index;
        }

        //allow to drag over the grid
        private void DragDropService_PreviewDragOver(object sender, RadDragOverEventArgs e)
        {
            if (e.DragInstance != null)
            {
                e.CanDrop = e.HitTarget is GridTableElement || e.HitTarget is GridDataRowElement;
            }
        }

        public class Item
        {
            public string UniqueIdentifier { get; set; }

            public string Title { get; set; }

            public Item(string uniqueIdentifier, string title)
            {
                this.UniqueIdentifier = uniqueIdentifier;
                this.Title = title;
            }
        }
    }
}